from database import db, app
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.app_web.web_model_factory import BlueprintDateReportedFactory
from flask_covid19.blueprints.data_owid.owid_model import OwidDateReported, OwidData, OwidContinent, OwidCountry
from flask_covid19.blueprints.data_owid.owid_model_import import OwidImport


class OwidServiceUpdateBase:
    def __init__(self, database, config: BlueprintConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OWID Service Update [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OWID Service Update [ready]")


class OwidServiceUpdateFull(OwidServiceUpdateBase):

    def __full_update_date_reported(self):
        app.logger.info(" WhoServiceUpdateFull.__full_update_date_reported [begin]")
        app.logger.info("------------------------------------------------------------")
        OwidDateReported.remove_all()
        i = 0
        log_lines = []
        for i_date_reported, in OwidImport.get_dates():
            i += 1
            o = BlueprintDateReportedFactory.create_new_object_for_owid(my_date_rep=i_date_reported)
            db.session.add(o)
            output = " full update  OWID date_reported [ " + str(i) + " ] " + i_date_reported + " added"
            log_lines.append(output)
        for log_line in log_lines:
            app.logger.info(log_line)
        db.session.commit()
        app.logger.info("")
        app.logger.info(" WhoServiceUpdateFull.__full_update_date_reported [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_continent(self):
        app.logger.info(" OwidServiceUpdate.__full_update_continent [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info("")
        log_lines = []
        OwidContinent.remove_all()
        i = 0
        for oi in OwidImport.get_all_continents():
            my_OwidContinent = OwidContinent(
                region=oi.continent,
                processed_update=False,
                processed_full_update=False,
            )
            db.session.add(my_OwidContinent)
            i += 1
            output = " full update OWID continent:  [ " + str(i) + " ] " + str(my_OwidContinent) + " added"
            log_lines.append(output)
        for log_line in log_lines:
            app.logger.info(log_line)
        db.session.commit()
        app.logger.info(" OwidServiceUpdate.__full_update_continent [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_country(self):
        app.logger.info(" OwidServiceUpdate.__full_update_country [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info("")
        OwidData.remove_all()
        OwidCountry.remove_all()
        self.__full_update_continent()
        log_lines = []
        i = 0
        for continent in OwidContinent.get_all():
            log_lines.append("continent.region: " + continent.region)
            for oi in OwidImport.get_countries(continent.region):
                i += 1
                o = OwidCountry(
                    continent_id=continent.id,
                    continent=continent,
                    location=oi.location,
                    iso_code=oi.iso_code,
                    population=oi.population,
                    population_density=oi.population_density,
                    median_age=oi.median_age,
                    aged_65_older=oi.aged_65_older,
                    aged_70_older=oi.aged_70_older,
                    gdp_per_capita=oi.gdp_per_capita,
                    extreme_poverty=oi.extreme_poverty,
                    cardiovasc_death_rate=oi.cardiovasc_death_rate,
                    diabetes_prevalence=oi.diabetes_prevalence,
                    female_smokers=oi.female_smokers,
                    male_smokers=oi.male_smokers,
                    handwashing_facilities=oi.handwashing_facilities,
                    hospital_beds_per_thousand=oi.hospital_beds_per_thousand,
                    life_expectancy=oi.life_expectancy,
                    human_development_index=oi.human_development_index,
                    processed_update=False,
                    processed_full_update=False,
                )
                db.session.add(o)
                output = " full update OWID country: [ " + str(i) + " ] " + str(o) + " added"
                log_lines.append(output)
        for log_line in log_lines:
            app.logger.info(log_line)
        db.session.commit()
        app.logger.info(" OwidServiceUpdate.__full_update_country [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __full_update_fact_table(self):
        app.logger.info(" OwidServiceUpdate.__full_update_fact_table [begin]")
        app.logger.info("------------------------------------------------------------")
        anzahl_db_zeilen_persistent = 0
        anzahl_db_zeilen_transient = 0
        lfd_nr_tage = 0
        OwidData.remove_all()
        for my_owid_date_reported in OwidDateReported.get_all():
            for oi in OwidImport.get_for_one_day(my_owid_date_reported.date_reported_import_str):
                pers_owid_country = OwidCountry.get_by_iso_code_and_location(
                    iso_code=oi.iso_code,
                    location=oi.location
                )
                o = OwidData(
                    date_reported=my_owid_date_reported,
                    country=pers_owid_country,
                    total_cases=oi.total_cases,
                    new_cases=oi.new_cases,
                    new_cases_smoothed=oi.new_cases_smoothed,
                    total_deaths=oi.total_deaths,
                    new_deaths=oi.new_deaths,
                    new_deaths_smoothed=oi.new_deaths_smoothed,
                    total_cases_per_million=oi.total_cases_per_million,
                    new_cases_per_million=oi.new_cases_per_million,
                    new_cases_smoothed_per_million=oi.new_cases_smoothed_per_million,
                    total_deaths_per_million=oi.total_deaths_per_million,
                    new_deaths_per_million=oi.new_deaths_per_million,
                    new_deaths_smoothed_per_million=oi.new_deaths_smoothed_per_million,
                    reproduction_rate=oi.reproduction_rate,
                    icu_patients=oi.icu_patients,
                    icu_patients_per_million=oi.icu_patients_per_million,
                    hosp_patients=oi.hosp_patients,
                    hosp_patients_per_million=oi.hosp_patients_per_million,
                    weekly_icu_admissions=oi.weekly_icu_admissions,
                    weekly_icu_admissions_per_million=oi.weekly_icu_admissions_per_million,
                    weekly_hosp_admissions=oi.weekly_hosp_admissions,
                    weekly_hosp_admissions_per_million=oi.weekly_hosp_admissions_per_million,
                    new_tests=oi.new_tests,
                    total_tests=oi.total_tests,
                    total_tests_per_thousand=oi.total_tests_per_thousand,
                    new_tests_per_thousand=oi.new_tests_per_thousand,
                    new_tests_smoothed=oi.new_tests_smoothed,
                    new_tests_smoothed_per_thousand=oi.new_tests_smoothed_per_thousand,
                    positive_rate=oi.positive_rate,
                    tests_per_case=oi.tests_per_case,
                    tests_units=oi.tests_units,
                    total_vaccinations=oi.total_vaccinations,
                    people_vaccinated=oi.people_vaccinated,
                    people_fully_vaccinated=oi.people_fully_vaccinated,
                    new_vaccinations=oi.new_vaccinations,
                    new_vaccinations_smoothed=oi.new_vaccinations_smoothed,
                    total_vaccinations_per_hundred=oi.total_vaccinations_per_hundred,
                    people_vaccinated_per_hundred=oi.people_vaccinated_per_hundred,
                    people_fully_vaccinated_per_hundred=oi.people_fully_vaccinated_per_hundred,
                    new_vaccinations_smoothed_per_million=oi.new_vaccinations_smoothed_per_million,
                    stringency_index=oi.stringency_index,
                    processed_update=False,
                    processed_full_update=False,
                )
                db.session.add(o)
                anzahl_db_zeilen_persistent += 1
                anzahl_db_zeilen_transient += 1
            my_owid_date_reported.set_processed()
            db.session.add(my_owid_date_reported)
            lfd_nr_tage += 1
            if lfd_nr_tage % 7 == 0:
                db.session.commit()
                app.logger.info(" full update OWID  "
                                + str(my_owid_date_reported) + " ... "
                                + str(anzahl_db_zeilen_persistent)
                                + " rows ( " + str(anzahl_db_zeilen_transient) + " )")
                anzahl_db_zeilen_transient = 0
        db.session.commit()
        app.logger.info(" full update OWID  :  "+str(anzahl_db_zeilen_persistent) +" rows total - for "+str(lfd_nr_tage)+" days")
        app.logger.info(" OwidServiceUpdate.__full_update_fact_table [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def full_update_dimension_tables(self):
        OwidData.remove_all()
        self.__full_update_date_reported()
        self.__full_update_country()
        return self

    def full_update_fact_table(self):
        app.logger.info(" WhoServiceUpdateFull.update_fact_table_initial_only [begin]")
        app.logger.info("------------------------------------------------------------")
        # TODO #216 implement OwidServiceUpdate.update_fact_table_initial_only()
        OwidData.remove_all()
        self.__full_update_fact_table()
        app.logger.info(" WhoServiceUpdateFull.update_fact_table_initial_only [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def full_update_star_schema(self):
        app.logger.info(" WhoServiceUpdateFull.full_update_star_schema [begin]")
        app.logger.info("------------------------------------------------------------")
        # TODO #218 implement OwidServiceUpdate.update_star_schema_initial()
        OwidData.remove_all()
        self.full_update_dimension_tables()
        self.full_update_fact_table()
        app.logger.info(" WhoServiceUpdateFull.full_update_star_schema [done]")
        app.logger.info("------------------------------------------------------------")
        return self


class OwidServiceUpdate(OwidServiceUpdateBase):

    def __get_new_dates_reported_from_import(self):
        todo = []
        owid_date_reported_all = []
        for i in OwidDateReported.get_all():
            owid_date_reported_all.append(i.date_reported_import_str)
        for datum_from_import in OwidImport.get_datum_of_all_import():
            if datum_from_import.date not in owid_date_reported_all:
                todo.append(datum_from_import[0])
        return todo

    def __get_new_continents_from_import(self):
        todo = []
        owid_continent_all = []
        for i in OwidContinent.get_all():
            owid_continent_all.append(i.region)
        for oi in OwidImport.get_all_continents():
            item = oi.continent
            if item not in owid_continent_all:
                todo.append(item)
        return todo

    def __get_new_countries_from_import(self):
        todo = []
        owid_countries = []
        for oc in OwidCountry.get_all():
            oc_key = (
                oc.iso_code,
                oc.location,
                oc.continent.region
            )
            owid_countries.append(oc_key)
        for oi in OwidImport.get_all_countries():
            country = (
                oi.iso_code,
                oi.location,
                oi.continent
            )
            if country not in owid_countries:
                todo.append(country)
        return todo

    def __update_date_reported(self):
        app.logger.info(" OwidServiceUpdate.__update_date_reported [begin]")
        app.logger.info("------------------------------------------------------------")
        i = 0
        log_lines = []
        for i_date_reported in self.__get_new_dates_reported_from_import():
            # app.logger.info(i_date_reported)
            i += 1
            o = OwidDateReported.create_new_object_factory(my_date_rep=i_date_reported)
            db.session.add(o)
            output = " added OwidDateReported: [ " + str(i) + " ] " + str(o) + " "
            log_lines.append(output)
        for log_line in log_lines:
            app.logger.info(log_line)
        db.session.commit()
        app.logger.info("")
        app.logger.info(" OwidServiceUpdate.__update_date_reported [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_continent(self):
        app.logger.info(" OwidServiceUpdate.__update_continent [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info("")
        log_lines = []
        for continent in self.__get_new_continents_from_import():
            my_owid_continent = OwidContinent(region=continent)
            db.session.add(my_owid_continent)
            log_lines.append("added OwidContinent: "+str(my_owid_continent))
        for log_line in log_lines:
            app.logger.info(log_line)
        db.session.commit()
        app.logger.info(" OwidServiceUpdate.__update_continent [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_country(self):
        app.logger.info(" OwidServiceUpdate.__update_country [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info("")
        self.__update_continent()
        for ci in self.__get_new_countries_from_import():
            iso_code = ci[0]
            location = ci[1]
            continent = ci[2]
            # app.logger.info("iso_code: " + iso_code + " - location: " + location + " - continent: " + continent)
            oi = OwidImport.get_country_for(iso_code=iso_code, location=location)
            owid_continent = OwidContinent.find_by_region(continent)
            o = OwidCountry(
                continent=owid_continent,
                location=oi.location,
                iso_code=oi.iso_code,
                population=oi.population,
                population_density=oi.population_density,
                median_age=oi.median_age,
                aged_65_older=oi.aged_65_older,
                aged_70_older=oi.aged_70_older,
                gdp_per_capita=oi.gdp_per_capita,
                extreme_poverty=oi.extreme_poverty,
                cardiovasc_death_rate=oi.cardiovasc_death_rate,
                diabetes_prevalence=oi.diabetes_prevalence,
                female_smokers=oi.female_smokers,
                male_smokers=oi.male_smokers,
                handwashing_facilities=oi.handwashing_facilities,
                hospital_beds_per_thousand=oi.hospital_beds_per_thousand,
                life_expectancy=oi.life_expectancy,
                human_development_index=oi.human_development_index
            )
            db.session.add(o)
            app.logger.info("added OwidCountry: " + str(o) + " ")
        db.session.commit()
        app.logger.info(" OwidServiceUpdate.__update_country [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def __update_fact_table(self):
        app.logger.info(" WhoServiceUpdateFull.__update_fact_table [begin]")
        app.logger.info("------------------------------------------------------------")
        anzahl_db_zeilen_persistent = 0
        anzahl_db_zeilen_transient = 0
        lfd_nr_tage = 0
        for unprocessed_owid_date_reported in OwidDateReported.get_unprocessed():
            unprocessed_owid_date_reported.set_processed()
            app.logger.info("unprocessed_date: " + str(unprocessed_owid_date_reported))
            for oi in OwidImport.get_for_one_day(unprocessed_owid_date_reported.date_reported_import_str):
                owid_country = OwidCountry.find_by_iso_code_and_location(
                    iso_code=oi.iso_code,
                    location=oi.location
                )
                o = OwidData(
                    date_reported=unprocessed_owid_date_reported,
                    country=owid_country,
                    total_cases=oi.total_cases,
                    new_cases=oi.new_cases,
                    new_cases_smoothed=oi.new_cases_smoothed,
                    total_deaths=oi.total_deaths,
                    new_deaths=oi.new_deaths,
                    new_deaths_smoothed=oi.new_deaths_smoothed,
                    total_cases_per_million=oi.total_cases_per_million,
                    new_cases_per_million=oi.new_cases_per_million,
                    new_cases_smoothed_per_million=oi.new_cases_smoothed_per_million,
                    total_deaths_per_million=oi.total_deaths_per_million,
                    new_deaths_per_million=oi.new_deaths_per_million,
                    new_deaths_smoothed_per_million=oi.new_deaths_smoothed_per_million,
                    reproduction_rate=oi.reproduction_rate,
                    icu_patients=oi.icu_patients,
                    icu_patients_per_million=oi.icu_patients_per_million,
                    hosp_patients=oi.hosp_patients,
                    hosp_patients_per_million=oi.hosp_patients_per_million,
                    weekly_icu_admissions=oi.weekly_icu_admissions,
                    weekly_icu_admissions_per_million=oi.weekly_icu_admissions_per_million,
                    weekly_hosp_admissions=oi.weekly_hosp_admissions,
                    weekly_hosp_admissions_per_million=oi.weekly_hosp_admissions_per_million,
                    new_tests=oi.new_tests,
                    total_tests=oi.total_tests,
                    total_tests_per_thousand=oi.total_tests_per_thousand,
                    new_tests_per_thousand=oi.new_tests_per_thousand,
                    new_tests_smoothed=oi.new_tests_smoothed,
                    new_tests_smoothed_per_thousand=oi.new_tests_smoothed_per_thousand,
                    positive_rate=oi.positive_rate,
                    tests_per_case=oi.tests_per_case,
                    tests_units=oi.tests_units,
                    total_vaccinations=oi.total_vaccinations,
                    people_vaccinated=oi.people_vaccinated,
                    people_fully_vaccinated=oi.people_fully_vaccinated,
                    new_vaccinations=oi.new_vaccinations,
                    new_vaccinations_smoothed=oi.new_vaccinations_smoothed,
                    total_vaccinations_per_hundred=oi.total_vaccinations_per_hundred,
                    people_vaccinated_per_hundred=oi.people_vaccinated_per_hundred,
                    people_fully_vaccinated_per_hundred=oi.people_fully_vaccinated_per_hundred,
                    new_vaccinations_smoothed_per_million=oi.new_vaccinations_smoothed_per_million,
                    stringency_index=oi.stringency_index,
                )
                db.session.add(o)
                anzahl_db_zeilen_persistent += 1
                anzahl_db_zeilen_transient += 1
            lfd_nr_tage += 1
            if lfd_nr_tage % 7 == 0:
                db.session.commit()
                app.logger.info(
                    " update OWID  :  added OwidData " + str(unprocessed_owid_date_reported)
                    + " ... " + str(anzahl_db_zeilen_persistent)
                    + " rows ( " + str(anzahl_db_zeilen_transient)
                    + " )")
                anzahl_db_zeilen_transient = 0
        db.session.commit()
        app.logger.info(" update OWID  :  added OwidData "
                        + str(anzahl_db_zeilen_persistent)
                        + " rows total - for "+str(lfd_nr_tage)+" days")
        app.logger.info(" WhoServiceUpdateFull.__update_fact_table [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_dimension_tables(self):
        app.logger.info(" OwidServiceUpdate.update_dimension_tables [begin]")
        app.logger.info("------------------------------------------------------------")
        # TODO #214 implement OwidServiceUpdate.update_dimension_tables_only()
        OwidData.remove_all()
        self.__update_date_reported()
        self.__update_country()
        app.logger.info(" OwidServiceUpdate.update_dimension_tables [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_fact_table(self):
        app.logger.info(" OwidServiceUpdate.update_fact_table_incremental_only [begin]")
        app.logger.info("------------------------------------------------------------")
        # TODO #215 implement OwidServiceUpdate.update_fact_table_incremental_only()
        self.__update_fact_table()
        app.logger.info(" OwidServiceUpdate.update_fact_table_incremental_only [done]")
        app.logger.info("------------------------------------------------------------")
        return self

    def update_star_schema(self):
        app.logger.info(" OwidServiceUpdate.update_star_schema [begin]")
        app.logger.info("------------------------------------------------------------")
        # TODO #217 implement OwidServiceUpdate.update_star_schema_incremental()
        OwidData.remove_all()
        self.__update_date_reported()
        self.__update_country()
        self.__update_fact_table()
        app.logger.info(" OwidServiceUpdate.update_star_schema [done]")
        app.logger.info("------------------------------------------------------------")
        return self
