from sqlalchemy import and_

from database import db, ITEMS_PER_PAGE
from flask_covid19.blueprints.app_all.all_model import BlueprintDateReported, BlueprintLocationGroup
from flask_covid19.blueprints.app_all.all_model import BlueprintLocation, BlueprintFactTable


class RkiMeldedatum(BlueprintDateReported):
    __tablename__ = 'rki_date_reported'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint(
            'date_reported_import_str',
            'datum',
            'year_day_of_year',
            name="uix_rki_cases_date_reported"
        ),
    )

    id = db.Column(db.Integer, primary_key=True)
    #
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year_day_of_year = db.Column(db.String(255), nullable=False, unique=True)
    year_month = db.Column(db.String(255), nullable=False)
    year_week = db.Column(db.String(255), nullable=False)
    #
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    week_of_year = db.Column(db.Integer, nullable=False)
    #
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)


class RkiBundesland(BlueprintLocationGroup):
    __tablename__ = 'rki_landkreis_bundesland'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('id_bundesland', 'location_group', name="uix_rki_landkreis_bundesland"),
    )

    def __str__(self):
        return " " + self.location_group + " "

    id = db.Column(db.Integer, primary_key=True)
    id_bundesland = db.Column(db.String(255), nullable=False)
    location_group = db.Column(db.String(255), nullable=False)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)


class RkiLandkreis(BlueprintLocation):
    __tablename__ = 'rki_landkreis'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('location', 'id_landkreis', name="uix_rki_landkreis"),
    )

    def __str__(self):
        return " " + str(self.location_group) + " : " + self.location + " ( " + self.id_landkreis + " ) "

    id = db.Column(db.Integer, primary_key=True)
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)
    location = db.Column(db.String(255), nullable=False)
    location_group_id = db.Column(db.Integer, db.ForeignKey('rki_landkreis_bundesland.id'), nullable=False)
    location_group = db.relationship(
        'RkiBundesland',
        lazy='joined',
        cascade='save-update',
        order_by='RkiBundesland.location_group')
    id_landkreis = db.Column(db.String(255), nullable=False)

    @classmethod
    def get_bochum(cls):
        return db.session.query(cls)\
            .filter(cls.location == 'SK Bochum')\
            .one()

    @classmethod
    def find_by_location_group(cls, location_group: RkiBundesland, page: int):
        return db.session.query(cls).filter(
            cls.location_group == location_group
        ).order_by(cls.location).all()

    @classmethod
    def get_by_location_group(cls, location_group: RkiBundesland, page: int):
        return db.session.query(cls).filter(
            cls.location_group == location_group
        ).order_by(cls.location).paginate(page, per_page=ITEMS_PER_PAGE)


class RkiData(BlueprintFactTable):
    __tablename__ = 'rki'
    __mapper_args__ = {'concrete': True}

    id = db.Column(db.Integer, primary_key=True)
    #
    date_reported_id = db.Column(db.Integer, db.ForeignKey('rki_date_reported.id'), nullable=False)
    date_reported = db.relationship(
        'RkiMeldedatum',
        lazy='joined',
        cascade='save-update',
        order_by='desc(RkiMeldedatum.datum)')
    location_id = db.Column(db.Integer, db.ForeignKey('rki_landkreis.id'), nullable=False)
    location = db.relationship(
        'RkiLandkreis',
        lazy='joined',
        cascade='save-update',
        order_by='asc(RkiLandkreis.location)')
    processed_update = db.Column(db.Boolean, nullable=False)
    processed_full_update = db.Column(db.Boolean, nullable=False)
    #
    fid = db.Column(db.String(255), nullable=False)
    altersgruppe = db.Column(db.String(255), nullable=False)
    geschlecht = db.Column(db.String(255), nullable=False)
    anzahl_fall = db.Column(db.String(255), nullable=False)
    anzahl_todesfall = db.Column(db.String(255), nullable=False)
    # meldedatum = db.Column(db.String(255), nullable=False)
    datenstand = db.Column(db.String(255), nullable=False)
    neuer_fall = db.Column(db.String(255), nullable=False)
    neuer_todesfall = db.Column(db.String(255), nullable=False)
    ref_datum = db.Column(db.String(255), nullable=False)
    neu_genesen = db.Column(db.String(255), nullable=False)
    anzahl_genesen = db.Column(db.String(255), nullable=False)
    ist_erkrankungsbeginn = db.Column(db.String(255), nullable=False)
    altersgruppe2 = db.Column(db.String(255), nullable=False)
